-- Version
IF
  (
   SELECT COUNT(*)
     FROM [version]
     WHERE [version] = 7
           AND [update] = 3
           AND [release] = 0
  ) = 0
    BEGIN
        INSERT INTO [version]
        ([version],
         [update],
         [release],
         [stable]
        )
        VALUES
        (7,
         3,
         0,
         0
        );
    END;
GO

-- User connections
CREATE TABLE [user_connections](
     [connection_id] [int] IDENTITY(1,1) NOT NULL,
     [license_id] [int] NOT NULL,
     [database_id] [int] NOT NULL,
     [database_name] [nvarchar](1024) NULL,
     [user] [nvarchar](1024) NULL,
     [password] [nvarchar](max) NULL,
     [windows_authentication] [bit] NULL,
     [different_schema] [bit] NULL,
     [connection_type] [nvarchar](100) NULL,
     [host] [nvarchar](1024) NULL,
     [port] [int] NULL,
     [service_name] [nvarchar](100) NULL,
     [network_alias] [nvarchar](100) NULL,
     [filter] [nvarchar](max) NULL,
     [multiple_schemas] [bit] NULL,
     [source_id] [int] NULL,
     [ssl_key_path] [nvarchar](500) NULL,
     [ssl_cert_path] [nvarchar](500) NULL,
     [ssl_ca_path] [nvarchar](500) NULL,
     [ssl_cipher] [nvarchar](max) NULL,
     [creation_date] [datetime] NOT NULL,
     [created_by] [nvarchar](1024) NULL,
     [last_modification_date] [datetime] NOT NULL,
     [modified_by] [nvarchar](50) NULL,
 CONSTRAINT [PK_user_connections] PRIMARY KEY CLUSTERED 
(
     [connection_id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
GRANT DELETE ON [user_connections] TO [users] AS [dbo];
GO
GRANT INSERT ON [user_connections] TO [users] AS [dbo];
GO
GRANT SELECT ON [user_connections] TO [users] AS [dbo];
GO
GRANT UPDATE ON [user_connections] TO [users] AS [dbo];
GO

ALTER TABLE [user_connections] ADD  CONSTRAINT [DF_user_connections_creation_date]  DEFAULT (getdate()) FOR [creation_date]
GO

ALTER TABLE [user_connections] ADD  CONSTRAINT [DF_user_connections_created_by]  DEFAULT (suser_sname()) FOR [created_by]
GO

ALTER TABLE [user_connections] ADD  CONSTRAINT [DF_user_connections_last_modification_date]  DEFAULT (getdate()) FOR [last_modification_date]
GO

ALTER TABLE [user_connections] ADD  CONSTRAINT [DF_user_connections_modified_by]  DEFAULT (suser_sname()) FOR [modified_by]
GO

ALTER TABLE [user_connections]  WITH CHECK ADD  CONSTRAINT [FK_user_connections_databases] FOREIGN KEY([database_id])
REFERENCES [databases] ([database_id])
GO

ALTER TABLE [user_connections] CHECK CONSTRAINT [FK_user_connections_databases]
GO

ALTER TABLE [user_connections]  WITH CHECK ADD  CONSTRAINT [FK_user_connections_licenses] FOREIGN KEY([license_id])
REFERENCES [licenses] ([license_id])
GO

ALTER TABLE [user_connections] CHECK CONSTRAINT [FK_user_connections_licenses]
GO

/****** Object:  Index [UK_user_connections_license_id_database_id]    Script Date: 08.03.2019 07:49:03 ******/
CREATE UNIQUE NONCLUSTERED INDEX [UK_user_connections_license_id_database_id] ON [user_connections]
(
     [license_id] ASC,
     [database_id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
GO

-- =============================================
-- Author:		Pawe Kwarciski
-- Create date:     2019-03-08
-- Description:	Updates last_modification_date, modified_by user_connections on insert or update
-- =============================================
CREATE TRIGGER [trg_user_connections_Modify]
   ON  [user_connections]
   AFTER INSERT,UPDATE
AS 
BEGIN
     UPDATE [user_connections]
     SET  last_modification_date = GETDATE(),
           modified_by = suser_sname()
     WHERE connection_id IN (SELECT DISTINCT connection_id FROM Inserted)
END
GO

ALTER TABLE [user_connections] ENABLE TRIGGER [trg_user_connections_Modify]
GO

-- Databases
IF EXISTS
         (
          SELECT *
            FROM [sys].[columns]
            WHERE object_id = OBJECT_ID(N'[databases]')
                  AND [name] = 'password'
         )
    BEGIN
        ALTER TABLE [databases] ALTER COLUMN [password] nvarchar(max);
    END;
GO

-- Version
UPDATE [version]
  SET
      [stable] = 1
  WHERE [version] = 7
        AND [update] = 3
        AND [release] = 0;
GO